@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1996, 2001 the Free Software Foundation, Inc.
+ * Copyright (C) 1996, 2001, 2004, 2005 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -20,18 +20,26 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #define initstate gawk_initstate
 #define setstate gawk_setstate
 #define random gawk_random
 #define srandom gawk_srandom
 
+#if SIZEOF_UNSIGNED_INT == 4
+typedef unsigned int gawk_uint32_t;
+typedef          int gawk_int32_t;
+#else
+#if SIZEOF_UNSIGNED_LONG == 4
+typedef unsigned long gawk_uint32_t;
+typedef          long gawk_int32_t;
+#endif
+#endif
+#define uint32_t gawk_uint32_t
+#define int32_t  gawk_int32_t
+
 #ifdef __STDC__
 #undef __P
 #define __P(s) s
